import os
import time

import base.base_service.constants

import version


class BaseService(object):
    def __init__(self, unit_name=None, interface_modules=[]):
        self._unit_name = unit_name or os.environ.get('UNIT_NAME')
        self._spawn_date = int(time.time())
        self._interface_modules = interface_modules

    def getName(self):
        return self._unit_name

    def getVersion(self):
        return version

    def getStatus(self):
        return base.base_service.constants.status.ALIVE

    def aliveSince(self):
        return self._spawn_date

    def getInterfaceVersion(self):
        def reduce_versions(acc, module):
            acc[module.__name__.split('.')[0]] = module.base_version
            return acc

        return reduce(reduce_versions, self._interface_modules)



import contextlib
import datetime.datetime
import tornado.gen
import statsd


class Client(object):
    def __init__(self, statsd_url, namespace):
        ip, port = statsd_url.split(':')
        self.client = statsd.StatsClient(ip, port=int(port), prefix=namespace)

    def timing(name, duration):
        pass

    @contextlib.contextmanager
    def instrument_manager(self, name):
        def send_duration():
            duration = (datetime.datetime.now() - start).total_seconds()
            self.timing("{}.duration".format(name),
                               int(duration * 1000))

        start = datetime.datetime.now()

        try:
            yield
        except Exception as e:
            self.incr('{}.exceptions.{}'.format(name, e.__class__.__name__))
            send_duration()

            raise e
        else:
            self.incr('{}.success'.format(name))
            send_duration()

    def instrument_decorator(self, name):
        def instrument_wrapper(fn):
            @tornado.gen.coroutine
            def fn_wrapper(*args, **kwargs):
                with self.instrument_wrapper(name):
                    result = yield tornado.gen.maybe_future(
                        fn(*args, **kwargs))
                    raise tornado.gen.Return(result)

            return fn_wrapper
        return instrument_wrapper



import socket
import opbeat


class Client(object):
    def __init__(self, organization_id, app_id, secret_token, hostname=None,
                 extra={}):
        self.extra = extra
        self.base_service = None
        self._client = opbeat.Client(
            organization_id=organization_id, app_id=app_id,
            secret_token=secret_token,
            hostname=hostname or socket.gethostname())

    def _build_base_extra(self):
        extras = {}

        if self.base_service:
            versions = self.base_service.getInterfaceVersion()
            versions.update({
                self.base_service.getName(): self.base_service.getVersion()})
        else:
            versions = {}

        for name, version in versions.iteritems():
            if version.git_version:
                gitver = version.git_version
                extras.update(
                    {'{}_git_version'.format(name): '{}/commit/{}'.format(
                        gitver.remote, gitver.commit)})
            if version.semantic_version:
                semver = version.semantic_version
                extras.update(
                    {'{}_semantic_version'.format(name): '{}.{}.{}'.format(
                        semver.major, semver.minor, semver.patch)})

        return extras

    def capture_exception(self, *args, **kwargs):
        kwargs['extra'] = dict(
            kwargs.get('extra', {}), **dict(
                self.extra, **self._build_base_extra()))

        self._client.capture_exception(*args, **kwargs)



import sys
import os
import opbeat_wrapper
import noop

if os.environ.get('OPBEAT_APP_ID', None):
    client = opbeat_wrapper.Client(
        os.environ.get('OPBEAT_ORGANIZATION_ID'),
        os.environ.get('OPBEAT_APP_ID'),
        os.environ.get('OPBEAT_SECRET_TOKEN'),
        os.environ.get('UNIT_NAME'))
else:
    client = noop.Client()


def notify_event(*args, **kwargs):
    if sys.exc_info()[0]:
        client.capture_exception(*args, **kwargs)



import os
import base.version.ttypes


def build_git_version():
    if os.environ.get('GIT_COMMIT'):
        return base.version.ttypes.GitVersion(
            os.environ.get('GIT_COMMIT'),
            os.environ.get('GIT_REMOTE'),
            os.environ.get('GIT_BRANCH'))
    else:
        return None


def build_semver_version():
    if os.environ.get('SEMVER_VERSION'):
        major, minor, patch = os.environ.get('SEMVER_VERSION').split('.')
        return base.version.ttypes.SemanticVersion(
            int(major[1:]), int(minor), int(patch))
    else:
        return None


version = base.version.ttypes.Version(build_semver_version(),
                                      build_git_version())



import os
import logging
import tornado.log

default_level = 'WARNING'

channel = logging.StreamHandler()
channel.setFormatter(tornado.log.LogFormatter(color=False))

logger = logging.getLogger()
logger.addHandler(channel)

logger.setLevel(
    logging.getLevelName(
        os.environ.get('LOGGER_LEVEL', default_level).upper()))



import socket
import opbeat


class Client(object):
    def __init__(self, organization_id, app_id, secret_token, hname=None,
                 extra={}):
        self.extra = extra
        self.base_service = None
        machine = hname or socket.gethostname()
        self._client = opbeat.Client(
            organization_id=organization_id, app_id=app_id,
            secret_token=secret_token, hostname=machine)

        def _build_base_extra(self):
            extras = {}
            versions = self.base_service.getInterfaceVersion()
            versions.update({
                self.base_service.getName(): self.base_service.getVersion()})

            for name, version in versions.iteritems():
                if version.git_version:
                    gitver = version.git_version
                    extras.update(
                        {'{}_git_version'.format(name): '{}/commit/{}'.format(
                            gitver.remote, gitver.commit)})
                if version.semantic_version:
                    semver = version.semantic_version
                    extras.update(
                        {'{}_semantic_version'.format(name): '{}.{}.{}'.format(
                            semver.major, semver.minor, semver.patch)})

            return extras

        def capture_exception(*args, **kwargs):
            kwargs['extra'] = dict(kwargs.get('extra', {}), **self._extra)
            self._client.capture_exception(*args, **kwargs)



import upfluence.log
import opbeat_wrapper


class Client(opbeat_wrapper.Client):
    def __init__(self):
        self.base_service = None

    def capture_exception(self, *args, **kwargs):
        upfluence.log.logger.error('exception:', exc_info=True)
        upfluence.log.logger.error(
            dict(self._build_base_extra(), **kwargs.get('extra', {})))
